;
; Originally designed and written by Daryl Rictor (c)2003-2004
;
; Modified by Grant Searle 2007-13
;
; Modified by Bernd Ulmann, August 2014
;

;  Define Registers and Port Pins
;   Acc = R00, R01 (results of multiply command)
.def    Acc                 =   R0      ; Reserved - maths accumulator for multiply operations
.def    Acch                =   R1

.def    nibbleTemp          =   R2      ; Temp storage for 4-bit (nibble) transfer      

.def    configByte          =   R3      ; As read from PORT D at startup
.set    CONFIG_PAL          =   7       ; 0xxxxxxx = NTSC, 0xxxxxxx = PAL
.set    CONFIG_8_OR_4_BIT   =   6       ; x0xxxxxx = 4 bit, x1xxxxxx= 8 bit
.set    CONFIG_N_TWI        =   5       ; xx1xxxxx = 4/8 bit, xx0xxxxx= Two wire
.set    CONFIG_80_CHAR_PER_LINE = 4     ; xxx0xxxx = 40, xxx1xxxx=80
.set    CONFIG_SINGLE_HEIGHT=   3       ; xxxx1xxx = Normal, xxxx0xxx = Double height
.set    CONFIG_BOLD         =   2       ; xxxxx0xx = Normal, xxxxx1xx = Bold

.def    currLineAtt         =   R4      ; Reserved for scanline processing
.set    ATT_80_CHAR_PER_LINE=   0       ; xxxxxxx0 = 40, xxxxxxx1=80
.set    ATT_BOLD            =   1
.set    ATT_DBL_TOP         =   2       ; xxxx00xx = Normal, xxxx01xx = Double top row, xxxx10xx = Double bottom row
.set    ATT_DBL_BOTTOM      =   3
            
.def    activeLineAtt       =   R5      ; Font/graphic style currently being used
.set    ATT_DBL_HEIGHT      =   2
.set    ATT_GRAPHICS        =   7

.def    previousInpt        =   R6
.def    charUnderCursor     =   R7      ; character under the cursor
.def    lastline            =   R10
.def    line1               =   R11
.def    cmdVal              =   R12
.def    temp                =   R13     ; General purpose temp
.def    temp2               =   R14     ; General purpose temp
.def    temp3               =   R15     ; General purpose temp

;---------------------------------------------------------------------------------------
.def    J                   =   R16     ; scratch 8 bit reg
.def    K                   =   R17     ; scratch 8 bit reg
.def    column              =   R18     ; hor cursor pos  0-79
.def    row                 =   R19     ; vert cursor pos 0-24
.def    vline               =   R20     ;\ vertical display line counter 0-261
.def    vlineh              =   R21     ;/ hi byte of vline counter
.def    chrln               =   R22     ; display chr line 0 - 199
.def    inpt                =   R23     ; host input 
.def    cursorChar          =   R24     ; cursor character
.def    cursorClk           =   R25     ; the cursor blink timer

;    X pair = R26,27
;    Y pair = R28,29
.def    Y                   =   R28:R29
;    Z pair = R30,31
;
;  X pair used to point to SRAM by refresh routines
;  Y pair used to point to SRAM by HOST processor
;  Z pair used to point to Font data in Program Memory & scrolling

;PORT pin defs
.set    syncpin             =   0       ; output
.set    shldpin             =   1       ; output
.set    datapin             =   2       ; input
.set    ackpin              =   3       ; output

.set    attr_start_high     =   high(RAMEND - 48 + 1)   ; Start of attribute RAM area
.set    attr_start_low      =   low(RAMEND - 48 + 1)
.set    attr_end_high       =   high(RAMEND - 23 + 1)   ; End of attribute RAM area,
.set    attr_end_low        =   low(RAMEND - 23 + 1)    ; leaving room for the stack

.set    screen_start_high   =   high(RAMEND - SRAM_SIZE + 1)                ; Start of screen RAM
.set    screen_start_low    =   low(RAMEND - SRAM_SIZE + 1)

.set    sol_2_high          =   high(RAMEND - SRAM_SIZE + 1 + 0x50)         ; Start of second line
.set    sol_2_low           =   low(RAMEND - SRAM_SIZE + 1 + 0x50)
.set    sol_3_high          =   high(RAMEND - SRAM_SIZE + 1 + 2 * 0x50)     ; Start of third line
.set    sol_3_low           =   low(RAMEND - SRAM_SIZE + 1 + 2 * 0x50)

.set    eol_23_high         =   high(RAMEND - SRAM_SIZE + 1 + 23 * 0x50)    ; End of line 23
.set    eol_23_low          =   low(RAMEND - SRAM_SIZE + 1 + 23 * 0x50)
.set    eol_24_high         =   high(RAMEND - SRAM_SIZE + 1 + 24 * 0x50)    ; End of line 24
.set    eol_24_low          =   low(RAMEND - SRAM_SIZE + 1 + 24 * 0x50)
.set    eol_25_high         =   high(RAMEND - SRAM_SIZE + 1 + 25 * 0x50)    ; End of line 25
.set    eol_25_low          =   low(RAMEND - SRAM_SIZE + 1 + 25 * 0x50)

.set    char_null           =   0x00    ; Some special characters
.set    char_delete         =   0x7f
.set    char_space          =   0x20
.set    char_underscore     =   0x5f

.set    max_column          =   0x50
.set    max_row             =   0x19
.set    max_column_40       =   0x28
