;http://searle.hostei.com/grant/MonitorKeyboard/index.html
;/**********************************************************************************/
;/  NTSC/PAL Video Display (40/80 character x 25 line monochrome text/graphics)     /
;/                                                                                  /
;/  Original 40x25 text designed and written by Daryl Rictor (c)2003-2004           /
;/                                                                                  /
;/  Modified by Grant Searle 2007-2013                                              /
;/      - 80x25 or 40x25 display                                                    /
;/      - Handshaking/data transfer rewritten                                       /
;/      - 8bit and 4bit and I2C interface (with help from Dave Curran)              /
;/  Modified by I. Kloeckl 2014                                                     /
;/      - port to ATmega32 and 8bit CPM interface                                   /
;/  Code cleanup by Bernd Ulmann, August 2014                                       /
;/      - This program can be assembled with avra                                   /
;/**********************************************************************************/

; include program specific definitions and macros
.include  "m32def.inc"                  ; Required for ATmega32
.include  "defs.inc"

;---------------------------------------------------------------------------------------------------------------------------
; Standard ATmega32 jump vectors

    jmp RESET                           ; Reset Handler
    jmp EXT_INT0                        ; IRQ0 Handler
    jmp EXT_INT1                        ; IRQ1 Handler
    jmp EXT_INT2                        ; IRQ2 Handler
    jmp TIM2_COMP                       ; Timer2 Compare Handler
    jmp TIM2_OVF                        ; Timer2 Overflow Handler
    jmp TIM1_CAPT                       ; Timer1 Capture Handler
    jmp TIM1_COMPA                      ; Timer1 Compare A Handler
    jmp TIM1_COMPB                      ; Timer1 Compare B Handler
    jmp TIM1_OVF                        ; Timer1 Overflow Handler
    jmp TIM0_COMP                       ; Timer0 Compare Handler
    jmp TIM0_OVF                        ; Timer0 Overflow Handler
    jmp SPI_STC                         ; SPI Transfer Complete Handler
    jmp USART_RXC                       ; USART, RX Complete Handler
    jmp USART_UDRE                      ; USART, UDR Empty Handler
    jmp USART_TXC                       ; USART, TX Complete Handler
    jmp ADCC                            ; ADC Conversion Complete Handler
    jmp EE_RDY                          ; EEPROM Ready Handler
    jmp ANA_COMP                        ; Analog Comparator Handler
    jmp TWI                             ; 2-wire Serial Interface Handler
    jmp SPM_RDY                         ; Store Program Memory Ready Handler
    
;---------------------------------------------------------------------------------------------------------------------------
; unused vectors point to software reset code

EXT_INT0:
EXT_INT1:
EXT_INT2:
TIM2_COMP:
TIM2_OVF:
TIM1_CAPT:
;TIM1_COMPA: //used elsewhere
TIM1_COMPB:
TIM1_OVF:
TIM0_COMP:
TIM0_OVF:
SPI_STC:
USART_RXC:
USART_UDRE:
USART_TXC:
ADCC:
EE_RDY:
ANA_COMP:
TWI:
SPM_RDY:

;---------------------------------------------------------------------------------------------------------------------------
; Program start

RESET:                          
    CLI                                 ; disable interupts

; Initialize the I/O Ports
;   Port A is input from bus interface
    LDI     J, 0xFF             
    out     PORTA, J                    ; enable pullups
    LDI     J, 0x00
    out     DDRA, J
;   Port C is output to bus interface
    LDI     J, 0x00             
    out     PORTC, J                    ; set output to L
    LDI     J, 0xFF
    out     DDRC, J
;   Port D is output to shift register/PAL
    LDI     J, 0x00             
    out     PORTD, J                    ; set output to L
    LDI     J, 0xFF
    out     DDRD, J
;   Port B0 is output to PAL, B1 is output to shift register LD/SH, 
;   B3 is output to /acknowledge data
;   B2 is input for data transmission request
    LDI     J, 0b00001101               ; enable pullup for pin 2, set /ack to H
    out     PORTB, J            
    LDI     J, 0b00001011               ; pin 0,1,3 of port B is output
    out     DDRB, J         

; Initialize Timer 1 
;   OCR1A will hold the value of 1024 and be used to reset the counter and 
;   cause an IRQ that will start the HSYNC process
;   The counter will count each clock cycle (16MHz - no prescaling)
setPAL:
    ldi     J, 0x38                     ; 0x0138 Last vertical line 312
    mov     lastline, J
    ldi     J, 0x40                     ; first vertical line of active disp
    mov     line1, J

    ldi     J, 0x00                     ; set timer 1 to CTC-OCR1A (WGM=0100), normal port operation
    out     TCCR1A, J
    ldi     J, 0x09                     ; set timer 1 to clk=no prescaling  (bits 4:3: WGM13:WMG12)
    out     TCCR1B, J       

    ldi     J, 0x04                     ;  0x0400 = 1024 clocks per line        
    out     OCR1AH, J        
    ldi     J, 0x00                     ;  0x0400 = 1024 clocks per line        
    out     OCR1AL, J                   ; set Counter 1 to cycle from 0 to 1024 and IRQ

    ldi     J, 0x00                     ; clear timer 1
    out     TCNT1H, J
    out     TCNT1L, J

    ldi     J, 0x10                     ; allow OCR1A IRQ's
    out     TIMSK, J            

; Initialise other registers
    ldi     J,0x00
    out     UCSRB,J                     ; Turn off serial

    ldi     J, 0x80
    out     ACSR, J                     ; turn off comparator

    ldi     J, high(RAMEND)             ; set stack pointer to top of SRAM 
    out     SPH, J
    ldi     J, low(RAMEND)
    out     SPL, J

; Initialise Program Variables
    ldi     chrln, 0x00                 ; keeps track of font gen line (0-7)
    ldi     vlineh, 0x00                ; vertical line counter 0-262
    ldi     vline, 0x00                 ;           "
    ldi     YL, screen_start_low
    ldi     YH, screen_start_high       ; init y pointer
    ldi     column, 0x00                ; keeps track of horizontal chr pos (0-79)
    ldi     row, 0x00                   ; keeps track of vertical chr pos (0-24)
    ldi     cursorClk, 0x00             ; init cursor blink timer
    ld      charUnderCursor, Y          ; init cursor character
    ldi     cursorChar, char_underscore ; init cursor '_' character
    ldi     J,  0xFF    
    mov     previousInpt, J
    ldi     J,  0x00
    mov     cmdVal, J

;---------------------------------------------------------------------------------------------------------------------------
; Set default font
    ldi     J, 0x00
    ori     J, 0 << ATT_80_CHAR_PER_LINE
    ori     J, 0 << ATT_BOLD
    ori     J, 0 << ATT_DBL_HEIGHT
    mov     activeLineAtt, J

    rcall   init_screen                 ; fill RAM with initial screen

    SEI                                 ; Initialisation complete. Enable IRQ's

;---------------------------------------------------------------------------------------------------------------------------
; Main Loop 

main:
    cpi     cursorChar, 0x00            ; is the cursor turned on?
    breq    endOfCursor                 ; no, skip cursor code
    cpi     cursorClk, 0xff             ; is clk at 0xFF?
    breq    solidCursor                 ; yes - solid cursor (no blink)
    cpi     cursorClk, 0x1C             ; is it above 28 frames?
    brpl    blinkCursor                 ; if >25, show Cursor
    mov     inpt, charUnderCursor
    call    setYfromRowCol
    call    putCharAtY
    rjmp    endOfCursor         
solidCursor:
    mov     inpt, cursorChar
    call    setYfromRowCol
    call    putCharAtY
    rjmp    endOfCursor
blinkCursor:
    mov     inpt, cursorChar
    call    setYfromRowCol
    call    putCharAtY
    cpi     cursorClk, 0x37             ; is clock >56 
    brmi    endOfCursor                 ; no
    clr     cursorClk                   ; reset timer
endOfCursor:

    ; If /data pin is L, then new data is available at input bus
    in      J, PINB
    andi    J, 0x04
    brne    Main                        ; Nothing there...
    in      inpt, PINA
    in      K, PORTB                    ; Data read, so create a L pulse on ack bit
    ldi     J, 0xf7                     ; Set B3 to L
    and     K, J
    out     PORTB, K
    ldi     J, 0x08                     ; Set B3 back to H
    or      K, J
    out     PORTB, K

    rcall   processChar                 ; Process the host data
    rjmp    Main                        ; Repeat

; Write welcome message to display
init_screen:
    ldi     zh, high(welcome_message << 1)
    ldi     zl, low(welcome_message << 1)
    ldi     r16, 122
init_screen_loop:
    lpm     inpt, z+
    push    zh                          ; Unfortunately, processChar destroys most register contents...
    push    zl
    push    r16
    rcall   processChar
    pop     r16
    pop     zl
    pop     zh
    dec     r16
    brne    init_screen_loop
    ret
welcome_message:
; 122 characters all in all...
    .db     12, 14, 0, 15, 0, 24, 1, 0                      ; 8 Chars
    .db     "AVR-BASED VIDEOGENERATOR", 0x0a, 0x0d          ; 26 Characters
    .db     "    DARYL RICTOR, GRANT SEARLE", 0x0a, 0x0d    ; 32 Chars
    .db     "    INGO KLOECKL, BERND ULMANN", 0x0a, 0x0d    ; 32 Chars
    .db     "AUGUST 26TH, 2014 ", 0x0a, 0x0d                ; 20 Chars
    .db     0x0a, 0x0d, 0x0a, 0x0d                          ; 4 Chars

;---------------------------------------------------------------------------------------------------------------------------
; IRQ Service routine for the video line generation
TIM1_COMPA:                             ; IRQ service for OCR1A 
    .include "vidgen.inc"               ; file that holds the video generation code 
    reti                                ; done

;---------------------------------------------------------------------------------------------------------------------------
; include subroutines and fonts

.include "ProcChr.inc"

.include "CGANormalFont.inc"    
.include "CGANormalFontWide.inc"
.include "CGABoldfont.inc"
.include "CGABoldFontWide.inc"
.include "BlockFont.inc"
